public abstract class CodeGenerator {
  public Code makeIfStatement(Expression cond, Statement _then, Statement _else) {
    Code code = makeCode();
    
    code.add(makeCond(cond));
    code.add(makeIf());
    code.add(makeCode(then_));
    code.add(makeCode(else_));
    code.add(makeGoto());

    return code;
  }

  protected abstract Code makeCode(Statement stmt);
  protected abstract Code makeIf();
  protected abstract Code makeCond(Expression cond);
  protected abstract Code makeGoto(Statement target);
}

class I386Generator extends CodeGenerator {
  protected Code makeCode(Statement stmt) { return weird_intel_code; }
  protected Code makeIf() { return IFCHX_no_registers; }
  protected Code makeCond(Expression cond) { return weird_intel_code; }
  protected Code makeGoto(Statement target) { return new I386_GOTO(); }
}

class SPARCGenerator extends CodeGenerator {
  protected Code makeCode(Statement stmt) { return weird_sparc_code; }
  protected Code makeIf() { return IFICMP_opt; }
  protected Code makeCond(Expression cond) { return weird_sparc_code; }
  protected Code makeGoto(Statement target) { return new SPARC_GOTO(); }
}
